@@ -1,5 +1,3 @@
-from __future__ import annotations
-
 import getpass
 import os
 import socket
@@ -12,7 +10,7 @@ from . import __version__
 def get_cf_global_attrs(**attrs):
 
     if "history" not in attrs:
-        attrs["history"] = f"Created: {time.ctime(time.time())}"
+        attrs["history"] = "Created: {}".format(time.ctime(time.time()))
 
     if "insitution" not in attrs:
         attrs["institution"] = "CarbonPlan"
